@@ -14,8 +14,6 @@ exceptions in order to be reusable in other packages.
 """
 from decimal import Decimal
 
-from ..namespaces import XSD_NAMESPACE
-
 from .helpers import is_idrefs, QNAME_PATTERN
 from .atomic_types import xsd10_atomic_types, xsd11_atomic_types, AnyAtomicType
 from .untyped import UntypedAtomic
@@ -26,7 +24,7 @@ from .numeric import Float10, Float, Integer, Int, NegativeInteger, \
 from .string import NormalizedString, XsdToken, Name, NCName, NMToken, Id, \
     Idref, Language, Entity
 from .uri import AnyURI
-from .binary import Base64Binary, HexBinary
+from .binary import AbstractBinary, Base64Binary, HexBinary
 from .datetime import AbstractDateTime, DateTime10, DateTime, DateTimeStamp, \
     Date10, Date, GregorianDay, GregorianMonth, GregorianYear, GregorianYear10, \
     GregorianMonthDay, GregorianYearMonth, GregorianYearMonth10, Time, Timezone, \
@@ -82,7 +80,7 @@ ATOMIC_VALUES = {
     'duration': Duration.fromstring('P1MT1S'),
     'dayTimeDuration': DayTimeDuration.fromstring('P1DT1S'),
     'yearMonthDuration': YearMonthDuration.fromstring('P1Y1M'),
-    'QName': QName(XSD_NAMESPACE, 'xs:element'),
+    'QName': QName("http://www.w3.org/2001/XMLSchema", 'xs:element'),
     'anyURI': AnyURI('https://example.com'),
     'normalizedString': NormalizedString(' alpha  '),
     'token': XsdToken('a token'),
@@ -122,4 +120,4 @@ __all__ = ['xsd10_atomic_types', 'xsd11_atomic_types', 'ATOMIC_VALUES', 'XSD_BUI
            'Integer', 'NonPositiveInteger', 'NegativeInteger', 'Long', 'Int', 'Short',
            'Byte', 'NonNegativeInteger', 'PositiveInteger', 'UnsignedLong', 'UnsignedInt',
            'UnsignedShort', 'UnsignedByte', 'AnyURI', 'Notation', 'QName', 'BooleanProxy',
-           'DecimalProxy', 'DoubleProxy10', 'DoubleProxy', 'UntypedAtomic']
+           'DecimalProxy', 'DoubleProxy10', 'DoubleProxy', 'UntypedAtomic', 'AbstractBinary']
